/* header file for QueryThread() and WholeTableThread()
 * Written by:
 *	Paul D. Boyle
 *	Department of Chemistry
 *	Box 8204
 *	North Carolina State University
 *	Raleigh, NC 27695-8204
 *
 *	March 1998
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

/* PG_BUF_LEN is defined by PostgreSQL */
#define PG_BUF_LEN       8192
#define DB_HOST_LEN      80   /* max length for hostname string len */
#define DB_STD1_LEN      32   /* max length for some various Query members */
#define DB_STD2_LEN      16   /* length for some Query memberssome Query members  */

enum TableType { LATT,
                 UNIT_CELL,
                 FORM,
                 EXPER,
                 REFINE,
                 ATOM_COORDS,
                 BOND_LENGTH,
                 BOND_ANGLE,
                 N_TABLES };

/* here is a structure to get the tuple resulting from a query */

typedef struct tuple {
        char **item_name,
             **value;
        } Tuple;

typedef struct query {
        pid_t          pid;                        /* process' PID */
        int            msg_id;                     /* message queue identifier */
        enum TableType query_type;                 /* see enumerations above */
        char           query_buffer[PG_BUF_LEN];   /* buffer for SQL query */
        char           db_host[DB_HOST_LEN];       /* hostname of server to be searched */
        char           db_name[DB_STD1_LEN];       /* name of database */
        char           db_options[DB_STD1_LEN];    /* options to be passed to DB */
        char           db_port[DB_STD2_LEN];       /* string length for port number */
        char           db_tty[DB_STD2_LEN];         /* tty for debugging */
        char           db_kerb_realm[DB_STD1_LEN]; /* for Kerboros realm */
        } Query;


#define UNIT_CELL_SEQ "SELECT unit_cell.unit_cell_seq FROM unit_cell " \
                      "WHERE unit_cell.id = '%s';"

#define WHOLE_TABL "SELECT %s.* FROM %s, unit_cell " \
                   "WHERE unit_cell.unit_cell_seq = %s.unit_cell_seq " \
                   "AND unit_cell.id = '%s';"

#define STD_HEADER_HTML "Content-type: html/text\n\n"\
                        "<html>\n<head><title>%s</title></head>"\
                        "<body>\n <H1>%s</H1>\n"

#define STD_TRAILER_HTML "</body></html>\n"

#define STD_UNIT_CELL_FMT "<table border width=60%%>"\
        "<tr align=left>"\
            "<td rowspan=5 ><h2>Structure  %s</h2></td>"\
            "</tr>"\
        "<tr align=left>"\
            "<td><b>a</b> = %s</td>"\
            "<td><b>alpha</b> = %s</td>"\
        "</tr>"\
        "<tr align=left>"\
            "<td><b>b</b> = %s</td>"\
            "<td><b>beta</b> = %s</td>"\
        "</tr>"\
        "<tr align=left>"\
            "<td><b>c</b> = %s</td>"\
            "<td><b>gamma</b> = %s</td>"\
        "</tr>"\
        "<tr align=left>"\
        "<td align=left colspan=2><b>Unit Cell Volume</b> = %s</td>"\
        "</tr>"\
        "</table>"\
        "<hr>"\
        "</body>"\
        "</html>"

/* Prototypes */
Tuple *SearchDatabase( Query *q, int *nt, int *nf );
int QueryThread( void *query);
void WholeTableThread(void *query);
