/* header file for data structures and defines used for 
 * sinfo application.
 * Written by:
 *	Paul D. Boyle
 *	Department of Chemistry
 *	Box 8204
 *	North Carolina State University
 *	Raleigh, NC 27695-8204
 *
 *	March 1998
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

/* these 2 defines are used to decide whether program executes as CGI bin
 * or command line mode.  If argc != 1 then CLI_MODE, if argc == 1
 * then program runs as CGI binary.
 */
#define CLI_MODE       1   /* command line mode */
#define CGI_MODE       2   /* cgi binary mode */ 


#define FIELD_TOKEN_STR  "&"  /* for parsing with strtok() */

/* define an sprint()'able string which defines where the html
 * formatted final tables go.
 */
#define WEB_ROOT   "/pages"         /* defines place where html pages
                                     * go relative to the home directory.
                                     */
#define TABLE_HOME "/tables/%s/"    /*  %s takes user_name */

/* define the lengths of all of the char arrays for the 
 * typedefs in one place.
 */
#define SYMB_LEN         3
#define CRYST_FAM_LEN   16
#define SPACE_GROUP_LEN 12
#define ID_LEN          12
#define DATE_LEN        12
#define NAME_LEN        40
#define RAD_LEN         12
#define SCAN_MODE_LEN   24
#define VENDOR_LEN      80
#define MODEL_LEN       24
#define REFINE_LEN       4
#define MATRIX_LEN      16
#define WEIGHT_LEN      21
#define CIF_FILE_LEN    80
#define CRT_FILE_LEN    80
#define ATOM_NAME_LEN    8
#define SITE_SYMM_LEN    8

/* some data structures for query results */
typedef struct element {
        char   symbol[SYMB_LEN];
        int    atomic_number;
        double  coval_radius,
               atomic_weight;
        } Element;

typedef struct lattice {
        short    n_mol_cell;
        char     crystal_family[CRYST_FAM_LEN];
        char     lattice_type;
        char     space_group[SPACE_GROUP_LEN];
        } Lattice;

typedef struct unit_cell {
        char   id[ID_LEN];
        char   sample_date[DATE_LEN];
        char   principal[NAME_LEN];
        char   sample_orig[NAME_LEN];
        char   radiation[RAD_LEN];
        double  wavelength,
               a, b, c, alpha, beta, gamma,
               volume, reduced_vol,
               sigma_a, sigma_b, sigma_c,
               sigma_alpha, sigma_beta, sigma_gamma,
               sigma_volume;
        } UnitCell;

typedef struct formula {
        Element element;
        double count;
        } Formula;

typedef struct experimental {
        char  scan_mode[SCAN_MODE_LEN],
               vendor_id[VENDOR_LEN] ,
               model_id[MODEL_LEN];
        double  temperature;
        short  n_refl_unit_cell;
        double  theta_min_unit_cell, theta_max_unit_cell,
               crystal_size_max, crystal_size_med, crystal_size_min,
               crystal_density, F000, mu,
               transmission_min, transmission_max;
        int    reflns_measured, reflns_unique,
               reflns_observed;
        double  sigma_cutoff, theta_min, 
               theta_max, r_merge;
        } Experimental;

typedef struct refinement {
        char   refine_type[REFINE_LEN],
               matrix_type[MATRIX_LEN],
               weight_scheme[WEIGHT_LEN];
        double  weight_coeff;
        int    refine_reflns_incl,
                n_parameters;
        double  r_conventional, r_weighted,
               gof, shift_err, final_df_max,
               final_df_min, ext_coeff,
               sigma_ext, chiral_coeff,
               sigma_chiral;
        char   cif_file[CIF_FILE_LEN];
        char   crt_file[CRT_FILE_LEN];
        } Refinement;

typedef struct atom_coords {
        short  atomic_number;
        char   atom_name[ATOM_NAME_LEN];
        char   site_symmetry[SITE_SYMM_LEN];
        unsigned long refine_flags; /* adp[10]    1 if aniso, 0 if isotropic
                                        occ[9]    1 if fixed, 0 if refined
                                         x[8]     1 if fixed, 0 if refined
                                         y[7]     1 if fixed, 0 if refined
                                         z[6]     1 if fixed, 0 if refined
                                         u11[5]   1 if fixed, 0 if refined
                                         u22[4]   1 if fixed, 0 if refined 
                                         u33[3]   1 if fixed, 0 if refined
                                         u12[2]   1 if fixed, 0 if refined
                                         u13[1]   1 if fixed, 0 if refined 
                                         u23[0]   1 if fixed, 0 if refined
                                     */
        double occupancy, x, y, z,
               sigma_occupancy,
               sigma_x, sigma_y, sigma_z,
               u11, u22, u33, u12, u13, u23,
               sigma_u11, sigma_u22, sigma_u33,
               sigma_u12, sigma_u13, sigma_u23;
        } AtomCoords;

typedef struct bond_length {
        char  atom_1[ATOM_NAME_LEN];
        char  atom_2[ATOM_NAME_LEN];
        double dist;
        double esd;    /* the so-called standard uncertainty */
        } BondLength; 

typedef struct bond_angle {
        char  atom_1[ATOM_NAME_LEN];
        char  atom_2[ATOM_NAME_LEN];
        char  atom_3[ATOM_NAME_LEN];
        double angle;
        double esd;      /* the so-called standard uncertainty */
        } BondAngle; 

/* define some names for the variables which appear in the html 
 * form.
 */
/* first the names of the different types of queries */
#define QT_TABLES       "tables"
#define QT_RED_VOL      "r_vol"
#define QT_FORMULA      "form"
#define QT_CIF          "cif"
#define QT_INVESTIGATOR "invesigator"
#define QT_SAMPL_ORIG   "sample_orig"
#define QT_SQL_MODE     "sql_mode"

/* now the data need to specify the queries */
#define  SERVER        "server"
#define  DB_NAME       "database_name"
#define  USER_NAME     "user_name"
#define  QUERY         "query"
#define  STRUCT_NAME   "struct_name"
#define  R_VOL_LOWER   "r_vol_lower"
#define  R_VOL_UPPER   "r_vol_upper"
#define  COMPOSITION   "composition"
#define  STRUCT_NM_CIF "struct_name_cif"
#define  EMAIL_ADDR    "email_address"
#define  INVEST_NAME   "investigator_name"
#define  SMPL_ORG_NAME "sample_orig_name"
#define  SQL_COMMAND   "sql_command"

enum QueryType  { FINAL_TABLES,
                  REDUCED_VOLUME,
                  FORMULA,
                  EMAIL_CIF,
                  PRINCIPAL_INVEST,
                  SAMPLE_ORIGINATOR,
                  SQL_EXPERT_MODE,
		  FENCE };

typedef struct 	{
		  char *qtname;
		  enum QueryType qtype;
		} QueryMap;

/*
QueryMap qmap[] = {
		{QT_TABLES, FINAL_TABLES},
		{QT_RED_VOL, REDUCED_VOLUME},
		{QT_FORMULA, FORMULA},
		{QT_CIF, EMAIL_CIF},
		{QT_INVESTIGATOR, PRINCIPAL_INVEST},
		{QT_SAMPL_ORIG, SAMPLE_ORIGINATOR},
		{QT_SQL_MODE, SQL_EXPERT_MODE} };

*/
/* define the place where sendmail or other MTA lives on your system */
#define SENDMAIL "/usr/sbin/sendmail -t"

/* define some sprintf()-able SQL statements. */

#define REDUCED_VOL  "SELECT unit_cell.id, unit_cell.a, unit_cell.b, " \
         "unit_cell.c, unit_cell.alpha, unit_cell.beta, unit_cell.gamma, " \
         "unit_cell.volume  FROM unit_cell " \
         "WHERE unit_cell.reduced_vol > %f AND unit_cell.reduced_vol < %f;"

#define MAIL_CIF "SELECT refinement.cif_file FROM refinement, unit_cell "\
        "WHERE unit_cell.unit_cell_seq = refinement.unit_cell_seq "\
        "AND unit_cell.id = '%s';%s"  /* this second field is the
                                       * email address and needs to be
                                       * copied to a safe place and 
                                       * deleted from the SQL commmand
                                       * buffer before being sent to the
                                       * database.
                                       */  


#define INVESTIGATOR "SELECT unit_cell.id FROM unit_cell " \
                     "WHERE unit_cell.principal = '%s';"

#define SAMPLE_ORIG "SELECT unit_cell.id FROM unit_cell " \
                     "WHERE unit_cell.sample_orig = '%s';"

/* Use the V1 and V2 parts of FORMULA to construct the view. You must
   use as many of the V2s as you have atomic numbers in the subset to
   be searched for. Use the "OR" conditional to connect the V2s.  */

/* Use the pid identifier to construct the name of the view.  */

#define FORMULA_V1 "CREATE VIEW F_%d_V AS SELECT * FROM FORMULA WHERE ";

#define FORMULA_V2 "(atomic_number = %d and count = %d)";

/* Use the S1 through S5 parts of FORMULA to construct the select.
   You must use as many of the S2s, S4s, and S5s as you have atomic
   numbers in the subset to be searched for. Note the assumption in
   S3 about the value of the table alias.  */

#define FORMULA_S1 "SELECT UC.id from unit_cell UC";
#define FORMULA_S2 ", F_%d_V %c";
#define FORMULA_S3 " WHERE (UC.unit_cell_seq = a.unit_cell_seq)";
#define FORMULA_S4 " and (%c.unit_cell_seq = %c.unit_cell_seq)";
#define FORMULA_S5 " and (%c.atomic_number = %d)";

/* used in code from sinfo_util.c */
#define TOLERANCE ((double)0.0001)

/* global */
int run_mode;

/* sinfo_util.c */
char *FormulaString(Formula *form, int n_atom_types);
double CalcFormulaWeight(Formula *form, int n_atom_types);
double RelativeDifference(double a, double b);
int PositionalRefinementFlags(unsigned long flag);
int CalcSignificantFigures(double sigma, double *new_sig);
int IsAnisotropic( unsigned long flag );
double CalcEquivADP(double u11, double u22, double u33, double u12, double u13, double u23);
double CalcSigmaCOSAngle(double angle, double sigma_angle);
double CalcSigmaCellVolume(UnitCell *cell);
