/* header file for message queue IPC for sinfo.
 * Written by:
 *	Paul D. Boyle
 *	Department of Chemistry
 *	Box 8204
 *	North Carolina State University
 *	Raleigh, NC 27695-8204
 *
 *	March 1998
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

/* total number of atom types (Formula) that can 
 * be transferred in a message queue.  This should be 
 * enough.
 */
#define N_ATOM_TYPES_MAX  20


/* default permissions for message queue */
#define PERMS             0660 

/* enum for identifying message type.  These need to 
 * be cast to long's when placing them in struct *_msg
 * (see below)
 */
enum MesgType {
     LATT_MSG = 1,
     CELL_MSG = 2,
     FORM_MSG = 3,
     EXPR_MSG = 4,
     REFN_MSG = 5,
     ERR_MSG  = 6,
     SPEC_MSG = 7 }; /* special message to atomic coords for diagonalized Ueq */

#define N_MESG_TYPES 5  /* make for() loops as <= N_MESG_TYPES */


/* define message structures for the various data structures */

struct latt_msg {
       long type;
       size_t  n_bytes;
       Lattice latt_data;
       };

struct cell_msg {
       long type;
       size_t  n_bytes;
       UnitCell cell_data;
       };

struct form_msg {
       long type;
       size_t  n_bytes;
       Formula formula_data[N_ATOM_TYPES_MAX];
       };

struct expr_msg {
       long type;
       size_t  n_bytes;
       Experimental exper_data;
       };

struct refn_msg {
       long type;
       size_t  n_bytes;
       Refinement refine_data;
       };

/* defines the largest message needing to be passed.  This will normally
 * be N_ATOM_TYPES_MAX * sizeof(struct form_msg);
 */

#define QUEUE_SIZ (N_ATOM_TYPES_MAX * sizeof(struct form_msg) )
